@@ -18,8 +18,8 @@
 #include <iosfwd>
 #if (defined(NDEBUG) && !defined(RANGES_ENSURE_MSG)) || \
     (!defined(NDEBUG) && !defined(RANGES_ASSERT) && \
-     defined(__GNUC__) && !defined(__clang__) && \
-     (__GNUC__ < 5 || defined(__MINGW32__)))
+     ((defined(__GNUC__) && !defined(__clang__) && (__GNUC__ < 5 || defined(__MINGW32__))) || \
+      defined(_MSVC_STL_VERSION)))
 #include <cstdio>
 #include <cstdlib>
 
@@ -41,8 +41,9 @@ namespace ranges
 #endif
 
 #ifndef RANGES_ASSERT
-#if !defined(NDEBUG) && defined(__GNUC__) && !defined(__clang__) && \
-    (__GNUC__ < 5 || defined(__MINGW32__))
+#if !defined(NDEBUG) && \
+    ((defined(__GNUC__) && !defined(__clang__) && (__GNUC__ < 5 || defined(__MINGW32__))) || \
+     defined(_MSVC_STL_VERSION))
 #define RANGES_ASSERT(...) \
     static_cast<void>((__VA_ARGS__) ? void(0) : \
         ::ranges::detail::assert_failure(__FILE__, __LINE__, "assertion failed: " #__VA_ARGS__))
@@ -109,6 +110,13 @@ namespace ranges
 #ifdef _LIBCPP_VERSION
 #define RANGES_BEGIN_NAMESPACE_STD _LIBCPP_BEGIN_NAMESPACE_STD
 #define RANGES_END_NAMESPACE_STD _LIBCPP_END_NAMESPACE_STD
+#elif defined(_MSVC_STL_VERSION)
+#define RANGES_BEGIN_NAMESPACE_STD _STD_BEGIN
+#define RANGES_END_NAMESPACE_STD _STD_END
+#elif defined(_GLIBCXX_DEBUG)
+#ifndef RANGES_NO_STD_FORWARD_DECLARATIONS
+#define RANGES_NO_STD_FORWARD_DECLARATIONS
+#endif
 #else
 #define RANGES_BEGIN_NAMESPACE_STD namespace std {
 #define RANGES_END_NAMESPACE_STD }
@@ -142,7 +150,7 @@ namespace ranges
 #define RANGES_CXX_STD_11 201103L
 #define RANGES_CXX_STD_14 201402L
 #define RANGES_CXX_STD_17 201703L
-#define RANGES_CXX_THREAD_LOCAL_PRE_STANDARD 200000L // Arbitrarily number between 0 and C++11
+#define RANGES_CXX_THREAD_LOCAL_PRE_STANDARD 200000L // Arbitrary number between 0 and C++11
 #define RANGES_CXX_THREAD_LOCAL_11 RANGES_CXX_STD_11
 #define RANGES_CXX_THREAD_LOCAL_14 RANGES_CXX_THREAD_LOCAL_11
 #define RANGES_CXX_THREAD_LOCAL_17 RANGES_CXX_THREAD_LOCAL_14
@@ -164,30 +172,7 @@ namespace ranges
 #define RANGES_CXX_IF_CONSTEXPR_17 201606L
 
 #if defined(_MSC_VER) && !defined(__clang__)
-#if _MSC_VER >= 1900
-#ifndef RANGES_CXX_VARIABLE_TEMPLATES
-#define RANGES_CXX_VARIABLE_TEMPLATES RANGES_CXX_VARIABLE_TEMPLATES_14
-#endif
-#ifndef RANGES_CXX_ATTRIBUTE_DEPRECATED
-#define RANGES_CXX_ATTRIBUTE_DEPRECATED RANGES_CXX_ATTRIBUTE_DEPRECATED_14
-#endif
-#if !defined(RANGES_CXX_RANGE_BASED_FOR) && defined(_MSVC_LANG) && _MSVC_LANG > 201402
-#define RANGES_CXX_RANGE_BASED_FOR RANGES_CXX_RANGE_BASED_FOR_17
-#endif
-#ifndef RANGES_CXX_LIB_IS_FINAL
-#define RANGES_CXX_LIB_IS_FINAL RANGES_CXX_LIB_IS_FINAL_14
-#endif
-#ifndef RANGES_CXX_RETURN_TYPE_DEDUCTION
-#define RANGES_CXX_RETURN_TYPE_DEDUCTION RANGES_CXX_RETURN_TYPE_DEDUCTION_14
-#endif
-#ifndef RANGES_CXX_GENERIC_LAMBDAS
-#define RANGES_CXX_GENERIC_LAMBDAS RANGES_CXX_GENERIC_LAMBDAS_14
-#endif
-
-#else // _MSC_VER < 1900
-#error Unsupported version of Visual C++
-#endif // _MSC_VER switch
-
+#define RANGES_CXX_VER _MSVC_LANG
 #define RANGES_DIAGNOSTIC_PUSH __pragma(warning(push))
 #define RANGES_DIAGNOSTIC_POP __pragma(warning(pop))
 #define RANGES_DIAGNOSTIC_IGNORE_PRAGMAS __pragma(warning(disable:4068))
@@ -207,9 +192,11 @@ namespace ranges
 #define RANGES_DIAGNOSTIC_IGNORE_UNDEFINED_FUNC_TEMPLATE
 #define RANGES_DIAGNOSTIC_IGNORE_INCONSISTENT_OVERRIDE
 #define RANGES_DIAGNOSTIC_IGNORE_RANGE_LOOP_ANALYSIS
+#define RANGES_DIAGNOSTIC_IGNORE_DEPRECATED_DECLARATIONS RANGES_DIAGNOSTIC_IGNORE(4996)
 
 #else // ^^^ defined(_MSC_VER) ^^^ / vvv !defined(_MSC_VER) vvv
 // Generic configuration using SD-6 feature test macros with fallback to __cplusplus
+#define RANGES_CXX_VER __cplusplus
 #if defined(__GNUC__) || defined(__clang__)
 #define RANGES_PRAGMA(X) _Pragma(#X)
 #define RANGES_DIAGNOSTIC_PUSH RANGES_PRAGMA(GCC diagnostic push)
@@ -231,6 +218,7 @@ namespace ranges
 #define RANGES_DIAGNOSTIC_IGNORE_UNDEFINED_FUNC_TEMPLATE RANGES_DIAGNOSTIC_IGNORE("-Wundefined-func-template")
 #define RANGES_DIAGNOSTIC_IGNORE_INCONSISTENT_OVERRIDE RANGES_DIAGNOSTIC_IGNORE("-Winconsistent-missing-override")
 #define RANGES_DIAGNOSTIC_IGNORE_RANGE_LOOP_ANALYSIS RANGES_DIAGNOSTIC_IGNORE("-Wrange-loop-analysis")
+#define RANGES_DIAGNOSTIC_IGNORE_DEPRECATED_DECLARATIONS RANGES_DIAGNOSTIC_IGNORE("-Wdeprecated-declarations")
 #else
 #define RANGES_DIAGNOSTIC_PUSH
 #define RANGES_DIAGNOSTIC_POP
@@ -249,22 +237,22 @@ namespace ranges
 #define RANGES_DIAGNOSTIC_IGNORE_MISSING_BRACES
 #define RANGES_DIAGNOSTIC_IGNORE_UNDEFINED_FUNC_TEMPLATE
 #define RANGES_DIAGNOSTIC_IGNORE_INCONSISTENT_OVERRIDE
+#define RANGES_DIAGNOSTIC_IGNORE_DEPRECATED_DECLARATIONS
 #endif
 #endif // MSVC/Generic configuration switch
 
 #define RANGES_CXX_FEATURE_CONCAT2(y, z) RANGES_CXX_ ## y ## _ ## z
 #define RANGES_CXX_FEATURE_CONCAT(y, z) RANGES_CXX_FEATURE_CONCAT2(y, z)
-#define RANGES_CXX_FEATURE(x) RANGES_CXX_FEATURE_CONCAT(x, RANGES_CXX_STD_NAME)
 
-#if __cplusplus >= RANGES_CXX_STD_17
-#define RANGES_CXX_STD_NAME 17
+#if RANGES_CXX_VER >= RANGES_CXX_STD_17
 #define RANGES_CXX_STD RANGES_CXX_STD_17
-#elif __cplusplus >= RANGES_CXX_STD_14
-#define RANGES_CXX_STD_NAME 14
+#define RANGES_CXX_FEATURE(x) RANGES_CXX_FEATURE_CONCAT(x, 17)
+#elif RANGES_CXX_VER >= RANGES_CXX_STD_14
 #define RANGES_CXX_STD RANGES_CXX_STD_14
+#define RANGES_CXX_FEATURE(x) RANGES_CXX_FEATURE_CONCAT(x, 14)
 #else
-#define RANGES_CXX_STD_NAME 11
 #define RANGES_CXX_STD RANGES_CXX_STD_11
+#define RANGES_CXX_FEATURE(x) RANGES_CXX_FEATURE_CONCAT(x, 11)
 #endif
 
 #ifndef RANGES_CXX_STATIC_ASSERT
@@ -346,19 +334,16 @@ namespace ranges
 #endif
 #endif
 
-#ifndef RANGES_DISABLE_DEPRECATED_WARNINGS
+#if !defined(RANGES_DEPRECATED) && !defined(RANGES_DISABLE_DEPRECATED_WARNINGS)
 #if RANGES_CXX_ATTRIBUTE_DEPRECATED &&            \
    !((defined(__clang__) || defined(__GNUC__)) && \
      RANGES_CXX_STD < RANGES_CXX_STD_14)
 #define RANGES_DEPRECATED(MSG) [[deprecated(MSG)]]
 #elif defined(__clang__) || defined(__GNUC__)
 #define RANGES_DEPRECATED(MSG) __attribute__((deprecated(MSG)))
-#elif defined(_MSC_VER)
-#define RANGES_DEPRECATED(MSG) __declspec(deprecated(MSG))
-#else
-#define RANGES_DEPRECATED(MSG)
 #endif
-#else
+#endif
+#ifndef RANGES_DEPRECATED
 #define RANGES_DEPRECATED(MSG)
 #endif
 
@@ -390,36 +375,26 @@ namespace ranges
 #endif
 
 #ifndef RANGES_CXX_INLINE_VARIABLES
-
-#ifdef __cpp_inline_variables // TODO: fix this if SD-6 picks another name
+#ifdef __cpp_inline_variables
 #define RANGES_CXX_INLINE_VARIABLES __cpp_inline_variables
-#elif defined(__clang__) && \
-    (__clang_major__ > 3 || __clang_major__ == 3 && __clang_minor__ == 9) && \
-    __cplusplus > 201402L
-// TODO: remove once clang defines __cpp_inline_variables (or equivalent)
+#elif defined(__clang__) && (__clang_major__ == 3 && __clang_minor__ == 9) && \
+    RANGES_CXX_VER > RANGES_CXX_STD_14
+// Clang 3.9 supports inline variables in C++17 mode, but doesn't define __cpp_inline_variables
 #define RANGES_CXX_INLINE_VARIABLES RANGES_CXX_INLINE_VARIABLES_17
 #else
 #define RANGES_CXX_INLINE_VARIABLES RANGES_CXX_FEATURE(INLINE_VARIABLES)
 #endif  // __cpp_inline_variables
-
 #endif  // RANGES_CXX_INLINE_VARIABLES
 
 #if RANGES_CXX_INLINE_VARIABLES < RANGES_CXX_INLINE_VARIABLES_17
-#define RANGES_INLINE_VARIABLE(type, name)                              \
-    inline namespace function_objects                                   \
-    {                                                                   \
-        inline namespace                                                \
-        {                                                               \
-            constexpr auto &name = ::ranges::static_const<type>::value; \
-        }                                                               \
+#define RANGES_INLINE_VARIABLE(type, name)                          \
+    inline namespace                                                \
+    {                                                               \
+        constexpr auto &name = ::ranges::static_const<type>::value; \
     }
-
 #else  // RANGES_CXX_INLINE_VARIABLES >= RANGES_CXX_INLINE_VARIABLES_17
 #define RANGES_INLINE_VARIABLE(type, name) \
-    inline namespace function_objects      \
-    {                                      \
-        inline constexpr type name{};      \
-    }
+    inline constexpr type name{};
 #endif // RANGES_CXX_INLINE_VARIABLES
 
 #ifndef RANGES_CXX_DEDUCTION_GUIDES
@@ -485,4 +460,15 @@ namespace ranges {
     }
 }
 
+#if !defined(RANGES_BROKEN_CPO_LOOKUP) && !defined(RANGES_DOXYGEN_INVOKED)
+#if defined(__clang__) // Workaround https://bugs.llvm.org/show_bug.cgi?id=37556
+#define RANGES_BROKEN_CPO_LOOKUP 1
+#elif defined(__GNUC__) && __GNUC__ < 6 // Workaround unknown GCC bug
+#define RANGES_BROKEN_CPO_LOOKUP 1
+#endif
+#endif
+#ifndef RANGES_BROKEN_CPO_LOOKUP
+#define RANGES_BROKEN_CPO_LOOKUP 0
+#endif
+
 #endif
